#!/usr/bin/env python

"""
Construct free/busy records for a user, either recording that user's own
availability schedule or the schedule of another user (using details provided
when scheduling events with that user).

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from bisect import bisect_left
from codecs import getwriter
from imiptools.data import get_window_end, Object
from imiptools.dates import get_default_timezone, to_utc_datetime
from imiptools.period import insert_period
from imiptools.profile import Preferences
from imip_store import FileStore, FilePublisher
import sys

# Main program.

if __name__ == "__main__":

    # Interpret the command line arguments.

    try:
        user = sys.argv[1]
        args = sys.argv[2:]
        participant = args and args[0] not in ("-n", "-s", "-v", "-r") and args[0] or user
        store_and_publish = "-s" in args
        include_needs_action = "-n" in args
        verbose = "-v" in args
        reset_updated_list = "-r" in args
    except IndexError:
        print >>sys.stderr, """\
    Need a user and an optional participant (if different from the user),
    along with the -s option if updating the store and the published details.
    """
        sys.exit(1)

    preferences = Preferences(user)
    tzid = preferences.get("TZID") or get_default_timezone()

    # Get the size of the free/busy window.

    try:
        window_size = int(preferences.get("window_size"))
    except (TypeError, ValueError):
        window_size = 100
    window_end = get_window_end(tzid, window_size)

    store = FileStore()
    publisher = FilePublisher()

    # Get identifiers for uncancelled events either from a list of events
    # providing free/busy periods at the end of the given time window, or from
    # a list of all events.

    all_events = not reset_updated_list and store.get_freebusy_providers(user, window_end)

    if not all_events:
        all_events = store.get_active_events(user)
        fb = []

    # With providers of additional periods, append to the existing collection.

    else:
        if user == participant:
            fb = store.get_freebusy(user)
        else:
            fb = store.get_freebusy_for_other(user, participant)

    # Obtain event objects.

    objs = []
    for uid, recurrenceid in all_events:
        if verbose:
            print >>sys.stderr, uid, recurrenceid
        event = store.get_event(user, uid, recurrenceid)
        if event:
            objs.append(Object(event))

    # Build a free/busy collection for the given user.

    for obj in objs:
        partstat = obj.get_participation_status(participant)
        recurrenceids = not obj.get_recurrenceid() and store.get_recurrences(user, obj.get_uid())

        if obj.get_participation(partstat, include_needs_action):
            for p in obj.get_active_periods(recurrenceids, tzid, window_end):
                fbp = obj.get_freebusy_period(p, partstat == "ORG")
                insert_period(fb, fbp)

    # Store and publish the free/busy collection.

    if store_and_publish:
        if user == participant:
            store.set_freebusy(user, fb)
            publisher.set_freebusy(user, fb)
            store.set_freebusy_providers(user, to_utc_datetime(window_end, tzid), [obj for obj in objs if obj.possibly_active_from(window_end, tzid)])
        else:
            store.set_freebusy_for_other(user, fb, participant)
    else:
        f = getwriter("utf-8")(sys.stdout)
        for item in fb:
            print >>f, "\t".join(item.as_tuple(strings_only=True))

# vim: tabstop=4 expandtab shiftwidth=4
